<?php

namespace MoSharePointObjectSync\Controller;

use MoSharePointObjectSync\Wrappers\pluginConstants;
use MoSharePointObjectSync\Wrappers\wpWrapper;

class appConfig{

    private static $instance;

    public static function getController(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
        }
        return self::$instance;
    }

    public function mo_sps_save_settings(){
        $option = sanitize_text_field($_POST['option']);
        switch ($option){
            case 'mo_sps_azure_config_option':{
                $this->mo_sps_save_azure_config();
                break;
            }
            case 'mo_sps__config_option':{
                $this->mo_sps_save_config_option();
                break;
            }
	        case 'mo_sps_delete_state':{
	            $this->mo_sps_delete_state();
	            break;
	        }
        }
    }

    public function mo_sps_delete_state(){
        check_admin_referer('mo_sps_delete_state');
	    wpWrapper::mo_sps_delete_option(pluginConstants::CURRENT_STATE);
	    wpWrapper::mo_sps_delete_option(pluginConstants::CURRENT_STATE_VALUE);
    }

    private function mo_sps_check_for_empty_or_null(&$input,$arr){
        foreach ($arr as $key){
            if(!isset($_POST[$key]) || empty($_POST[$key])){
                return false;
            }
            if($key == 'folder_path') $input[$key] = sanitize_text_field(rtrim($_POST[$key],'/'));
            else $input[$key] = sanitize_text_field($_POST[$key]);
        }
        return $input;
    }

    private function mo_sps_save_azure_config(){
        check_admin_referer('mo_sps_azure_config_option');
        $input_arr = ['client_id','client_secret','tenant_id'];
        $sanitized_arr = [];

        if(!$this->mo_sps_check_for_empty_or_null($sanitized_arr,$input_arr)){  
            wpWrapper::mo_sps__show_error_notice(esc_html__("Input is empty or present in the incorrect format."));
            return;
        }

            $sanitized_arr['client_secret'] = wpWrapper::mo_sps_encrypt_data($sanitized_arr['client_secret'],hash("sha256",$sanitized_arr['client_id']));
            $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
            if(empty($config)) $config = [];
            foreach($input_arr as $arr) {
                $config[$arr] = $sanitized_arr[$arr];
            }
            wpWrapper::mo_sps_set_option("mo_sps_application_config",$config);
            wpWrapper::mo_sps__show_success_notice(esc_html__("Settings Saved Successfully."));


    }

    private function mo_sps_save_config_option() {
        check_admin_referer('mo_sps__config_option');
        $sharepoint_folder_path_url = urldecode( esc_url( $_POST['sharepoint_folder_path_url'] ) );

        if(empty($sharepoint_folder_path_url)){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Input is empty or present in the incorrect format."));
            return;
        }

        $url_components = parse_url($sharepoint_folder_path_url);
        if(!checkdnsrr($url_components['host'])){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Invalid URL Domain!"));
            return;
        }

        if(strpos($url_components['host'],'sharepoint.com') === false ){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Invalid SharePoint Online URL!"));
            return;
        }

        $url_components['path'] = str_replace('Forms/AllItems.aspx',"",$url_components['path']);
        $url_components['folder_path'] = $url_components['path'];
        $url_components['site_part'] = '/';

        $is_sites_url = strpos($url_components['path'],'sites');
        $is_teams_url = strpos($url_components['path'],'teams');

        if(( $is_sites_url!== false ) || ( $is_teams_url !== false )){
            $site_part_arr = explode('/',$url_components['path'],4);
            if(!in_array($site_part_arr[1],['sites','teams'])){
                wpWrapper::mo_sps__show_error_notice(esc_html__("Invalid url provided."));
                return;
            }
            $url_components['site_part'] = '/'.implode('/',[$site_part_arr[1],$site_part_arr[2]]).'/';
            $url_components['folder_path'] = '/'.str_replace($url_components['site_part'],'',$url_components['path']);

            unset($url_components['path']);
        }

        if(isset($url_components['query'])){
            $folder_path = [];
            parse_str($url_components['query'],$folder_path);
            
            if(!isset($folder_path['id'])){
                wpWrapper::mo_sps__show_error_notice(esc_html__("Invalid SharePoint Online URL!"));
                return;
            }

            $folder_path = $folder_path['id'];
            $site_replace_part = $url_components['site_part'];
            if($site_replace_part === '/'){
                $folder_path = rtrim(ltrim($folder_path,'/'),'/');
            }else{
                $folder_path = str_replace($site_replace_part,'',$folder_path);
            }

            $url_components['folder_path'] = '/'.$folder_path;
            unset($url_components['query']);
        }

        if(isset($url_components['fragment']))
            unset($url_components['fragment']);


        $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);

        $config['sharepoint_folder_path_url'] = $sharepoint_folder_path_url;
        $config['admin_uri'] = $url_components['scheme'].'://'.$url_components['host'];
        $config['site_uri'] = rtrim($url_components['scheme'].'://'.$url_components['host'].$url_components['site_part'],'/');
        $config['folder_path'] = rtrim($url_components['folder_path'],'/');
        $config['sharepoint_permission_page_url'] = $url_components['scheme'].'://'. str_replace('.sharepoint.com','-admin.sharepoint.com',$url_components['host']).'/_layouts/15/appinv.aspx';

        $feedback_config = wpWrapper::mo_sps_get_option(pluginConstants::FEEDBACK_CONFIG);
        $feedback_config['folder_path'] = $config['folder_path'];
        $feedback_config['admin_uri'] = $config['admin_uri'];
        $feedback_config['site_uri'] = $config['site_uri'];
        wpWrapper::mo_sps_set_option("mo_sps_feedback_config", $feedback_config);
        
        wpWrapper::mo_sps_set_option("mo_sps_application_config",$config);
        wpWrapper::mo_sps_delete_option(pluginConstants::CURRENT_STATE);
        wpWrapper::mo_sps_delete_option(pluginConstants::CURRENT_STATE_VALUE);
        wpWrapper::mo_sps__show_success_notice(esc_html__("Settings Saved Successfully."));
    }
}