<?php
/**
 * Admin Observer to carry out specific admin tasks.
 *
 * @package embed-sharepoint-onedrive-documents/Observer
 */

namespace MoSharePointObjectSync\Observer;

use MoSharePointObjectSync\API\Azure;
use MoSharePointObjectSync\API\CustomerMOSPS;
use MoSharePointObjectSync\Wrappers\pluginConstants;
use MoSharePointObjectSync\Wrappers\sharepointWrapper;
use MoSharePointObjectSync\Wrappers\wpWrapper;

/**
 * Class to handle ajax actions of the plugin.
 */
class adminObserver {

	/**
	 * Holds the adminObserver class instance.
	 *
	 * @var adminObserver
	 */
	private static $obj;

	/**
	 * Object instance(adminObserver) getter method.
	 *
	 * @return adminObserver
	 */
	public static function getObserver() {
		if ( ! isset( self::$obj ) ) {
			self::$obj = new adminObserver();
		}
		return self::$obj;
	}

	/**
	 * Function to execute specific actions based on option value in request.
	 *
	 * @return void
	 */
	public function mo_sps_admin_observer() {

		if ( ! isset( $_REQUEST['sps_option'] ) || !current_user_can('administrator') ) {
			return;
		}

		switch ( $_REQUEST['sps_option'] ) {
			case 'mo_sps_test_app_connection':
				$this->mo_sps_test_app_connection();
				break;
			case 'mo_sps_test_sharepoint_connection':
				$this->mo_sps_test_sharepoint_connection();
				break;
			case 'mo_sps_contact_us_query_option':
				$this->mo_sps_support_form_init( $_POST );
				break;
			case 'mo_sps_demo_request_option':
				$this->mo_sps_demo_request_init( $_POST );
				break;
			case 'mo_sps_feedback_option':
				$this->mo_sps_feedback_form_init( $_REQUEST, $_POST );
				break;
		}

	}

	/**
	 * Function to test sharepoint app connection for documents.
	 *
	 * @return void
	 */
	private function mo_sps_test_app_connection() {
		$config = wpWrapper::mo_sps_get_option( pluginConstants::APP_CONFIG );
		$client   = Azure::get_client( $config );
		$response = $client->mo_sps_test_azure_ad_connection();
		if ( $response['status'] ) {
			$this->mo_sps_display_azure_app_connection_success();

		} else {
            $feedback_config['test_configuration'] = 'error';
			$error_code = array(
				'Error'       => $response['data']['error'],
				'Description' => empty( $response['data']['error'] ) ? '' : $response['data']['error_description'],
			);

			if(strpos($error_code['Description'],"AADSTS700016") !== false ){
				$error_code["Solution"] = "Please copy the correct value of <b>Application (client) ID</b> from Overview section of <a href='https://portal.azure.com/'>App registrations application</a>";
			}else if(strpos($error_code['Description'],"AADSTS7000215") !== false ){
				$error_code["Solution"] = "Please copy the correct value of <b>Client Secret</b> from Certificates & secrets section of <a href='https://portal.azure.com/'>App registrations application</a>.";
			}else if(strpos($error_code['Description'],"AADSTS90002") !== false ){
				$error_code["Solution"] = "Please copy the correct value of <b>Directory (tenant) ID</b> from Overview section of <a href='https://portal.azure.com/'>App registrations application</a>.";
			}

			$this->mo_sps_display_error_message( $error_code );

		}
	}

	private function mo_sps_test_sharepoint_connection(){
		$config = wpWrapper::mo_sps_get_option( pluginConstants::APP_CONFIG );

		$feedback_config                       = wpWrapper::mo_sps_get_option( pluginConstants::FEEDBACK_CONFIG );
		$feedback_config['test_configuration'] = 'success';

		$client   = Azure::get_client( $config );

		if(!isset($config['folder_path'])){
			$error_code = array(
				'Error'       => 'Incomplete configurations.',
				'Description' => 'Step 2 is not completed.',
			);

			$this->mo_sps_display_error_message( $error_code );
		}

		$response = $client->mo_sps_document_array($config['folder_path']);
		if(isset($response['d'])){
			$this->mo_sps_reload_window_opener();
			$this->mo_sps_display_test_sharepoint_connection();
		}else{

			$error = json_encode($response);

			$error_code = array(
				'Error'       => $error,
			);

			if(strpos($error,'Token type is not allowed.') !== false){
				$error_code['Description'] = 'This error occurs because the new SharePoint subscription Grant App Permission is disabled by default.';
				$error_code['Solution'] = 'You can easily fix this error by granting permission by running PowerShell commands. To fix this issue please&nbsp;<a target="_blank" href="https://faq.miniorange.com/knowledgebase/token-type-is-not-allowed-error-on-sharepoint-rest-api/">Click Here</a>';
			}else if(strpos($error,'Forbidden') !== false ){
				$error_code['Description'] = 'Please check if you have granted tenant permissions for your SharePoint site with above permissions.';
			}else if(strpos($error,'File Not Found') !== false ){
				$error_code['Description'] = 'Folder path that you have configured in step 2 is not found at your SharePoint site.';
				$error_code['Solution'] = 'Please find the proper steps to configure SharePoint Folder Path URL in Step 2 from here: <a href="https://plugins.miniorange.com/microsoft-sharepoint-wordpress-integration#step1">[How to get SharePoint Site Folder Path?]</a>';
			}else{
				$error_code['Description'] = 'Azure AD App connection from Step 1 is not successful. Please click on Test Configurations button from Step 1 to see more about this issue.';
			}

			$this->mo_sps_display_error_message( $error_code );
			
		}
	}

	private function mo_sps_reload_window_opener(){
        echo '<script>window.opener.reload_page_to_see_reflected_changes();</script>';
    }

	/**
	 * Function to init support form request.
	 *
	 * @param array $post // contains $_POST.
	 * @return void
	 */
	private function mo_sps_support_form_init( $post ) {
		$submited = $this->mo_sps_send_support_query( $post );
		if ( ! is_null( $submited ) ) {
			if ( false === $submited ) {
				wpWrapper::mo_sps__show_error_notice( esc_html__( 'Your query could not be submitted. Please try again.' ) );
			} else {
				wpWrapper::mo_sps__show_success_notice( esc_html__( 'Thanks for getting in touch! We shall get back to you shortly.' ) );
			}
		}
	}

	/**
	 * Function to init demo request
	 *
	 * @param array $post // contains $_POST.
	 * @return void
	 */
	private function mo_sps_demo_request_init( $post ) {
		$submited = $this->mo_sps_send_demo_request_query( $post );
		if ( ! is_null( $submited ) ) {
			if ( false === $submited ) {
				wpWrapper::mo_sps__show_error_notice( esc_html__( 'Your query could not be submitted. Please try again.' ) );
			} else {
				wpWrapper::mo_sps__show_success_notice( esc_html__( 'Thanks for getting in touch! We shall get back to you shortly.' ) );
			}
		}
	}

	/**
	 * Function to init feedback form request.
	 *
	 * @param array $request // Holds $_REQUEST.
	 * @param array $post // Holds $_POST.
	 * @return void
	 */
	private function mo_sps_feedback_form_init( $request, $post ) {
		$sent     = isset( $request['miniorange_feedback_submit'] );
		$skip     = isset( $request['miniorange_skip_feedback'] );
		$submited = $this->mo_sps_send_email_alert( $post, $skip, $sent );
		if ( JSON_ERROR_NONE === json_last_error() ) {
			if ( is_array( $submited ) && array_key_exists( 'status', $submited ) && 'ERROR' === $submited['status'] ) {
				wpWrapper::mo_sps__show_error_notice( esc_html( $submited['message'] ) );
			} else {
				if ( false === $submited ) {
					wpWrapper::mo_sps__show_error_notice( esc_html__( 'Error while submitting the query.' ) );
				}
			}
		}

			include_once ABSPATH . 'wp-admin/includes/plugin.php';

			deactivate_plugins( MO_SPS_PLUGIN_FILE );
			wpWrapper::mo_sps__show_success_notice( esc_html__( 'Thank you for the feedback.' ) );
	}

	/**
	 * Function to send feedback email.
	 *
	 * @param array   $post // Holds $_POST.
	 * @param boolean $is_skipped // check if feedback form is skipped.
	 * @param boolean $is_sent // check if feedback form is sent.
	 * @return array
	 */
	private function mo_sps_send_email_alert( $post, $is_skipped = false, $is_sent = false ) {

		$user = wp_get_current_user();

		$message            = 'Plugin Deactivated';
		$deactivate_reasons = array_key_exists( 'sps_reason', $post ) ? $post['sps_reason'] : array();

		$deactivate_reason_message = array_key_exists( 'query_feedback', $post ) ? htmlspecialchars( $post['query_feedback'] ) : false;

		if ( $is_skipped && false === $deactivate_reason_message ) {
			$deactivate_reason_message = 'skipped';
		}
		if ( $is_sent && false === $deactivate_reason_message ) {
			$deactivate_reason_message = 'Send';
		}

		$reply_required = '';
		if ( isset( $post['get_reply'] ) ) {
			$reply_required = htmlspecialchars( $post['get_reply'] );
		}
		if ( empty( $reply_required ) ) {
			$reply_required = "don't reply";
			$message       .= '<b style="color:red";> &nbsp; [Reply :' . $reply_required . ']</b>';
		} else {
			$reply_required = 'yes';
			$message       .= '[Reply :' . $reply_required . ']';
		}

		if ( is_multisite() ) {
			$multisite_enabled = 'True';
		} else {
			$multisite_enabled = 'False';
		}

		$message .= ', [Multisite enabled: ' . $multisite_enabled . ']';

		$message .= ', Feedback : ' . $deactivate_reason_message . '';

		$email   = '';
		$reasons = '';

		foreach ( $deactivate_reasons as $reason ) {
			$reasons .= $reason;
			$reasons .= ',';
		}

		$reasons  = substr( $reasons, 0, -1 );
		$message .= ', [Reasons :' . $reasons . ']';

		if ( isset( $post['query_mail'] ) ) {
			$email = $post['query_mail'];
		}

		if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
			$email = get_option( 'mo_sps_admin_email' );
			if ( empty( $email ) ) {
				$email = $user->user_email;
			}
		}
		$phone            = get_option( 'mo_sps_admin_phone' );
		$feedback_reasons = new CustomerMOSPS();

		$response = json_decode( $feedback_reasons->mo_sps_send_email_alert( $email, $phone, $message ), true );

		return $response;

	}


	/**
	 * Function to send support form request
	 *
	 * @param array $post // Holds $_POST.
	 * @return array
	 */
	private function mo_sps_send_support_query( $post ) {
		$email = sanitize_email( $post['mo_sps_contact_us_email'] );
		$phone = htmlspecialchars( $post['mo_sps_contact_us_phone'] );
		$query = htmlspecialchars( $post['mo_sps_contact_us_query'] );

		$query = '[Embed sharepoint onedrive documents plugin] ' . $query;

		$CustomerMOSPS = new CustomerMOSPS();

		$response = $CustomerMOSPS->mo_sps_submit_contact_us( $email, $phone, $query );

		return $response;
	}

	/**
	 * Function to send demo request
	 *
	 * @param array $post // Holds $_POST.
	 * @return array
	 */
	private function mo_sps_send_demo_request_query( $post ) {
		$email = sanitize_email( $post['demo_email'] );
		$query = htmlspecialchars( $post['demo_description'] );

		$addons_selected = array();
		$addons          = pluginConstants::INTEGRATIONS_TITLE;
		foreach ( $addons as $key => $value ) {
			if ( isset( $post[ $key ] ) && 'true' === $post[ $key ] ) {
				$addons_selected[ $key ] = $value;
			}
		}

		$integrations_selected = implode( ', ', array_values( $addons_selected ) );

		$query = '[Demo Request For Embed sharepoint onedrive documents plugin] ' . $query;

		$CustomerMOSPS = new CustomerMOSPS();

		$response = $CustomerMOSPS->mo_sps_submit_demo_query( $email, '', $query, $integrations_selected );

		return $response;
	}

	/**
	 * Function to display error message.
	 *
	 * @param array $error_code // Contains error codes.
	 * @return void
	 */
	public function mo_sps_display_error_message( $error_code ) {
		$feedback_config                       = wpWrapper::mo_sps_get_option( pluginConstants::FEEDBACK_CONFIG );
		$feedback_config['test_configuration'] = 'failed';
		wpWrapper::mo_sps_set_option( 'mo_sps_feedback_config', $feedback_config );
		?> 

			<div style="width:100%;display:flex;flex-direction:column;justify-content:center;align-items:center;font-size:15px;margin-top:10px;width:100%;">
				<div style="width:86%;padding: 15px;text-align: center;background-color:#f2dede;color:#a94442;border: 1px solid #E6B3B2;font-size: 18pt;margin-bottom:20px;">
					Error
				</div>

				<table class="mo-ms-tab-content-app-config-table" style="border-collapse:collapse;width:90%">
					<tr>
						<td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" colspan="2"><h2><span>Test Configuration Failed</span></h2></td>
					</tr>
					<?php
					foreach ( $error_code as $key => $value ) {
						echo '<tr><td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" class="left-div"><span style="margin-right:10px;"><b>' . esc_html( $key ) . ':</b></span></td>
                       <td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" class="right-div"><span>' . ( $value ) . '</span></td></tr>';
					}
					?>
				</table>
				<h3 style="margin:20px;">
					Contact us at <a style="color:#dc143c" href="mailto:samlsupport@xecurify.com">samlsupport@xecurify.com</a>
				</h3>
			</div>
		<?php
		exit();
	}

	/**
	 * Function to display Azure AD app connection status
	 *
	 * @return void
	 */
	private function mo_sps_display_azure_app_connection_success() {
		?>
		<div style="width:100%;height:100%;display:flex;align-items:center;flex-direction:column;border:1px solid #eee;padding:10px;">
				<div style="width:90%;color: #3c763d;background-color: #dff0d8;padding: 2%;margin-bottom: 20px;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
					Success
				</div>
			<div style="display:block;text-align:center;margin-bottom:4%;">
				<svg class="animate" width="100" height="100">
					<filter id="dropshadow" height="">
					<feGaussianBlur in="SourceAlpha" stdDeviation="3" result="blur"></feGaussianBlur>
					<feFlood flood-color="rgba(76, 175, 80, 1)" flood-opacity="0.5" result="color"></feFlood>
					<feComposite in="color" in2="blur" operator="in" result="blur"></feComposite>
					<feMerge> 
						<feMergeNode></feMergeNode>
						<feMergeNode in="SourceGraphic"></feMergeNode>
					</feMerge>
					</filter>
					<circle cx="50" cy="50" r="46.5" fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"></circle>
					<path d="M67,93 A46.5,46.5 0,1,0 7,32 L43,67 L88,19" fill="none" stroke="rgba(76, 175, 80, 1)" stroke-width="5" stroke-linecap="round" stroke-dasharray="80 1000" stroke-dashoffset="-220" style="filter:url(#dropshadow)"></path>
				</svg>
			<div style="margin-top:1%;margin-left:2%;color:#3c763d;background-color:#dff0d8;">
			<div style="color: #3c763d;background-color: #dff0d8;padding: 2%;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
				Connected to Your Azure AD Application.
			</div>
                <div style="padding: 10px">
                    Please proceed to <button onclick="close_and_redirect_to_app_config_tab()" style="cursor:pointer;background:#dff0d8;border:0px;color: black"><b><u>Step 2 : Sharepoint configuration</u></b></button><br> section of the plugin to link the sharepoint site.
                </div>
			</div>
			</div>
				<style>
				svg.animate path {
					animation: dash 1.5s linear both;
					animation-delay: 1s;
				}
				@keyframes dash {
					0% { stroke-dashoffset: 210; }
					75% { stroke-dashoffset: -220; }
					100% { stroke-dashoffset: -205; }
				}
			</style>
		</div>
			</div>
		</div>

        <script>
            function close_and_redirect_to_app_config_tab(){
                window.opener.redirect_to_app_config_tab();
                self.close();
            }
        </script>
		<?php
		 $this->load_css();
		exit();
	}

	/**
	 * Function to display test app connection status
	 *
	 * @return void
	 */
	private function mo_sps_display_test_sharepoint_connection() {
		?>
		<div style="width:100%;height:100%;display:flex;align-items:center;flex-direction:column;border:1px solid #eee;padding:10px;">
				<div style="width:90%;color: #3c763d;background-color: #dff0d8;padding: 2%;margin-bottom: 20px;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
					Success
				</div>
			<div style="display:block;text-align:center;margin-bottom:4%;">
				<svg class="animate" width="100" height="100">
					<filter id="dropshadow" height="">
					<feGaussianBlur in="SourceAlpha" stdDeviation="3" result="blur"></feGaussianBlur>
					<feFlood flood-color="rgba(76, 175, 80, 1)" flood-opacity="0.5" result="color"></feFlood>
					<feComposite in="color" in2="blur" operator="in" result="blur"></feComposite>
					<feMerge> 
						<feMergeNode></feMergeNode>
						<feMergeNode in="SourceGraphic"></feMergeNode>
					</feMerge>
					</filter>
					<circle cx="50" cy="50" r="46.5" fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"></circle>
					<path d="M67,93 A46.5,46.5 0,1,0 7,32 L43,67 L88,19" fill="none" stroke="rgba(76, 175, 80, 1)" stroke-width="5" stroke-linecap="round" stroke-dasharray="80 1000" stroke-dashoffset="-220" style="filter:url(#dropshadow)"></path>
				</svg>
			<div style="margin-top:1%;margin-left:2%;color:#3c763d;background-color:#dff0d8;width:90%;">
			<div style="color: #3c763d;background-color: #dff0d8;padding: 2%;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
				Connected to your SharePoint Site.
			</div>
			</div>
			</div>
				<div style="margin:25%;margin-top:0%;display:flex;text-align:center;">

				<div style="margin-right:20px;">
				<input class="mo-ms-tab-content-button" style="box-shadow:none!important;height:30px;background-color: #1B9BA1;border-color: #1B9BA1;color: #FFF;cursor: pointer;" type="button" value="Preview Sharepoint Files/Folders" onClick="close_and_redirect_to_document_sync();"> &nbsp;
				</div>

				<div style="margin-right:20px;">
				<input class="mo-ms-tab-content-button" style="box-shadow:none!important;height:30px;background-color: #1B9BA1;border-color: #1B9BA1;color: #FFF;cursor: pointer;" type="button" value="Embed in Pages/Posts" onClick="close_and_redirect_to_shortcode_tab();">
				</div>
				
				</div>
				<style>
				svg.animate path {
					animation: dash 1.5s linear both;
					animation-delay: 1s;
				}
				@keyframes dash {
					0% { stroke-dashoffset: 210; }
					75% { stroke-dashoffset: -220; }
					100% { stroke-dashoffset: -205; }
				}
			</style>
		</div>


			</div>
		</div>
		<script>
			function close_and_redirect_to_sync_user(){
				window.opener.redirect_to_sync_users();
				self.close();
			}  
			function close_and_redirect_to_document_sync(){
				window.opener.redirect_to_document_sync();
				self.close();
			}  
			function close_and_redirect_to_shortcode_tab(){
				window.opener.redirect_to_shortcode_tab();
				self.close();
			}
		</script>

		<?php
		$this->load_css();
		exit();
	}

	/**
	 * Function to display sharepoint user profile attributes.
	 *
	 * @param array $details // Contains user profile attributes.
	 * @return void
	 */
	private function mo_sps_display_fetch_attributes( $details ) {
		?>

		<div style="display:flex;justify-content:center;align-items:center;flex-direction:column;border:1px solid #eee;padding:10px;">


				<div style="width:90%;color: #3c763d;background-color: #dff0d8;padding: 2%;margin-bottom: 20px;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
					Success
				</div>
				<div style="display:block;text-align:center;margin-bottom:4%;"><svg class="animate" width="100" height="100">
				<filter id="dropshadow" height="">
					<feGaussianBlur in="SourceAlpha" stdDeviation="3" result="blur"></feGaussianBlur>
					<feFlood flood-color="rgba(76, 175, 80, 1)" flood-opacity="0.5" result="color"></feFlood>
					<feComposite in="color" in2="blur" operator="in" result="blur"></feComposite>
					<feMerge> 
					<feMergeNode></feMergeNode>
					<feMergeNode in="SourceGraphic"></feMergeNode>
					</feMerge>
				</filter>
				<circle cx="50" cy="50" r="46.5" fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"></circle>
				<path d="M67,93 A46.5,46.5 0,1,0 7,32 L43,67 L88,19" fill="none" stroke="rgba(76, 175, 80, 1)" stroke-width="5" stroke-linecap="round" stroke-dasharray="80 1000" stroke-dashoffset="-220" style="filter:url(#dropshadow)"></path>
				</svg>
			<script>
				window.onunload = refreshParent;
				function refreshParent() {
					window.opener.location.reload();
				}
			</script>

			<style>
				svg.animate path {
					animation: dash 1.5s linear both;
					animation-delay: 1s;
				}

				@keyframes dash {
					0% { stroke-dashoffset: 210; }
					75% { stroke-dashoffset: -220; }
					100% { stroke-dashoffset: -205; }
				}
			</style></div>

				<div style="border-top:1px solid #eee;width:95%;"></div>

				<div class="test-container" style="margin-top:10px;background:#fff;">
					<table class="mo-ms-tab-content-app-config-table">
						<tr>
							<td style="text-align:center;" colspan="2">
								<span><h2>Test Attributes:</h2></span>
							</td>
						</tr>
				<?php
				foreach ( $details as $key => $value ) {
					if ( ! is_array( $value ) && ! empty( $value ) ) {
						?>
					<tr>
						<td class="left-div"><span><?php echo esc_html( $key ); ?></span></td>
						<td class="right-div"><span><?php echo esc_html( $value ); ?></span></td>
					</tr>
						<?php
					}
				}
				?>
					</table>
				</div>
		</div>
				</div>
		<?php
		$this->load_css();
		exit();
	}

	/**
	 * Function load css in test connection window.
	 *
	 * @return void
	 */
	private function load_css() {
		?>
		<style>
			.test-container{
				width: 100%;
				background: #f1f1f1;
				margin-top: -30px;
			}

			.mo-ms-tab-content-app-config-table{
				max-width: 1000px;
				background: white;
				padding: 1em 2em;
				margin: 2em auto;
				border-collapse:collapse;
				border-spacing:0;
				display:table;
				font-size:14pt;
			}

			.mo-ms-tab-content-app-config-table td.left-div{
				width: 40%;
				word-break: break-all;
				font-weight:bold;
				border:2px solid #949090;
				padding:2%;
			}
			.mo-ms-tab-content-app-config-table td.right-div{
				width: 40%;
				word-break: break-all;
				padding:2%;
				border:2px solid #949090;
				word-wrap:break-word;
			}

		</style>
		<?php
	}
}
