<?php

/*
Plugin Name: Embed SharePoint OneDrive Documents
Plugin URI: https://plugins.miniorange.com/
Description: This plugin will allow you to sync users/files/sites from SharePoint Online/Office 365 to wordpress.
Version: 1.1.3
Author: miniOrange
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/

namespace MoSharePointObjectSync;
require_once __DIR__ . '/vendor/autoload.php';

use MoSharePointObjectSync\View\adminView;
use MoSharePointObjectSync\Controller\adminController;
use MoSharePointObjectSync\Observer\adminObserver;
use MoSharePointObjectSync\Observer\documentObserver;
use MoSharePointObjectSync\Observer\shortcodeSharepoint;

use MoSharePointObjectSync\View\feedbackForm;
use MoSharePointObjectSync\Wrappers\wpWrapper;

define('MO_SPS_PLUGIN_FILE',__FILE__);
define('MO_SPS_PLUGIN_DIR',__DIR__.DIRECTORY_SEPARATOR);
define('MO_SPS_PLUGIN_URL',plugin_dir_url(__FILE__));
define('PLUGIN_VERSION','1.1.2');
define('MO_SPS_LICENSE_PLANS','https://plugins.miniorange.com/microsoft-sharepoint-wordpress-integration#pricing-cards');

class MOsps{

    private static $instance;
    public static $version = PLUGIN_VERSION;

    public static function mo_sps_load_instance(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
            self::$instance->mo_sps_load_hooks();
        }
        return self::$instance;
    }

    public function mo_sps_load_hooks(){

        add_action('admin_menu',[$this,'mo_sps_admin_menu']);
        add_action( 'admin_enqueue_scripts', [$this, 'mo_sps_settings_style' ] );
        add_action( 'admin_init', array ($this, 'mo_sp_redirect_after_activation') );
        add_action('admin_init',[adminController::getController(),'mo_sps_admin_controller']);
        add_action( 'admin_footer', [feedbackForm::getView() , 'mo_sps_display_feedback_form'] );
        add_action('init',[adminObserver::getObserver(),'mo_sps_admin_observer']);
     

	    register_activation_hook(__FILE__,array($this,'mo_sps_plugin_activate'));

        add_shortcode( 'mo_sps_Shortcode_SharePoint', [shortcodeSharepoint::getObserver(),'mo_sps_shortcode_document_observer'] );

        add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array($this,'settings_link'));   
        add_action( 'init',[$this,'mo_sps_gutenburg']);

        add_action( "wp_ajax_mysearchaction", [documentObserver::getObserver(),'mo_sps_document_search_observer']);
        add_action( "wp_ajax_mysyncaction", [documentObserver::getObserver(),'mo_sps_document_observer'] );


        add_action( 'admin_enqueue_scripts', array( $this, 'mo_sps_settings_script' ) );
        register_uninstall_hook(__FILE__, 'mo_sps_uninstall');
    }



    public function mo_sps_load_media_library_scripts()
    {
        global $pagenow;

        $mode = get_user_option('media_library_mode', get_current_user_id()) ? get_user_option('media_library_mode', get_current_user_id()) : 'grid';
        if ($mode === 'list' || $mode === 'grid') {
            $this->mo_sps_load_assets();
        }

        
    }

    public function mo_sps_load_assets()
    {
        wp_enqueue_script('jquery');

  
        wp_enqueue_style(
            'mo-sps-style',
            plugins_url('/includes/css/media.css', __FILE__),
            array(),
            PLUGIN_VERSION
        );

            wp_register_script(
                'mo-sps-base',
                plugins_url('/includes/js/media.js', __FILE__),
                array('jquery'),
                PLUGIN_VERSION
            );
            
            $params = [  
                'sharepoint_icon' => esc_url(plugin_dir_url(__FILE__).'/images/microsoft-sharepoint.svg'),
                'admin_uri' => admin_url(),
	            'license_plans' => MO_SPS_LICENSE_PLANS,
            ];
            wp_enqueue_script('mo-sps-base');
            wp_localize_script('mo-sps-base', 'mo_sps', $params);

  
    }

	public function mo_sps_plugin_activate(){
        wpWrapper::mo_sps_set_option("mo_sps_feedback_config",array());
		update_option('mo_sp_do_activation_redirect', true);
	}
    
    function mo_sp_redirect_after_activation() {
	    if (get_option('mo_sp_do_activation_redirect')) {
		    delete_option('mo_sp_do_activation_redirect');

		    if(!isset($_GET['activate-multi'])) {
			    wp_redirect(admin_url() . 'admin.php?page=mo_sps');
			    exit;
		    }
	    }
    }

    public function mo_sps_admin_menu(){
        $page = add_menu_page(
            'miniOrange SharePoint Integration ' .__('+ Sync'),
            'miniOrange SharePoint Integration',
            'administrator',
            'mo_sps',
            [adminView::getView(),'mo_sps_menu_display'],
            plugin_dir_url( __FILE__ ) . 'images/miniorange.png'
        );
    }

    function mo_sps_settings_style($page){

        global $pagenow;

        if ($pagenow === 'upload.php') {
            $this->mo_sps_load_media_library_scripts();
        }

        if( $page != 'toplevel_page_mo_sps'){
            return;
        }
        $css_url = plugins_url('includes/css/mo_sps_settings.css',__FILE__);
        $css_phone_url = plugins_url('includes/css/phone.css',__FILE__);
        $css_jquery_ui_url = plugins_url('includes/css/jquery-ui.css',__FILE__);
        $css_license_view_url = plugins_url('includes/css/license.css',__FILE__);

        wp_enqueue_style('mo_sps_css',$css_url,array(),self::$version);
        wp_enqueue_style('mo_sps_phone_css',$css_phone_url,array(),self::$version);
        wp_enqueue_style('mo_sps_jquery_ui_css',$css_jquery_ui_url,array(),self::$version);
        wp_enqueue_style('mo_sps_license_view_css',$css_license_view_url,array(),self::$version);
    }

    function mo_sps_settings_script($page){
        $phone_js_url = plugins_url('includes/js/phone.js',__FILE__);
        $setting_js_url= plugins_url('includes/js/settings.js',__FILE__);
        wp_enqueue_script('mo_sps_phone_js',$phone_js_url,array(),self::$version);
        wp_enqueue_script('mo_settings_js',$setting_js_url,array(),self::$version);

    }

    function so_enqueue_scripts(){
        wp_register_script( 
          'ajaxHandle', 
          plugins_url('includes/js/ajax.js', __FILE__), 
          array(), 
          false, 
          true 
        );
        wp_enqueue_script( 'ajaxHandle' );
        wp_localize_script( 
          'ajaxHandle', 
          'ajax_object', 
          array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) 
        );
      }

    function settings_link( $links ) {
        // Build and escape the URL.
        $url = esc_url( add_query_arg(
            'page',
            'mo_sps',
            get_admin_url() . 'admin.php'
        ) );
        // Create the link.
        $settings_link = "<a href='$url'>" . __( 'Settings' ) . '</a>';
        // Adds the link to the end of the array.
        array_push(
            $links,
            $settings_link
        );
        return $links;
    }
      function mo_sps_gutenburg()  {
        $src = plugins_url('includes/js/gutenburg-block.js',__FILE__);
        
        wp_register_script('custom-cta-js', $src, array('wp-blocks','wp-editor'), self::$version);

        
        if(isset($_GET['post']))
        {
            $post_id = $_GET['post'];
            $post_info = get_post( $post_id);
            $post_content = wp_strip_all_tags($post_info->post_content);
           
        wp_localize_script('custom-cta-js','post_content',$post_content);

            
    }

        register_block_type('sps/custom-cta',array(
             'editor_script' => 'custom-cta-js',

        ));

        
    }

}
MOsps::mo_sps_load_instance();